#/*
# * +--------------------------------------------------------------------------+
# * | Copyright 1999, 2002 Rational Software Corporation.  All Rights Reserved.|
# * | This software contains proprietary and confidential information of       |
# * | Rational and its suppliers.  Use, disclosure or reproduction is          |
# * | prohibited without the prior express written consent of Rational.        |
# * +--------------------------------------------------------------------------+
# */
############################################################################
#UpdateMSProjectData.pl
#
# This script determines whether a MS Project task complete and "on time" 
# or not and sets the "on time" value in the MS Project measure.  
#
############################################################################

use PjCLib;
use Win32::OLE;
require CQPerlExt;

use constant CURRENCY_SIGN => "\$";
use constant PERCENT_SIGN => "\%";
use constant CURRENCY_DEMARCATOR => "\,";
use constant LEFT_PAREN => "(";
use constant RIGHT_PAREN => ")";
use constant MINUS => "-";
use constant HOUR => " hr";
use constant HOURS => " hrs";
use constant DAY => " day";
use constant DAYS => " days";
use constant NULL_STRING => "";
use constant PROJECT_MEASURE_TABLE => "MSP_Task_Measure";
use constant PROJECT_AGGREGAGE_MEASURE_TABLE => "MSPAggregate_measure";
use constant DBID => "DBID";
use constant YES => "yes";
use constant NO => "no";
use constant BOOL_TRUE => "True";
use constant BOOL_FALSE => "False";


# Temp Fields
#   These fields hold temporary values that need to be transformed into actual values
use constant ACTUAL_COST_STR => "ActualCostStr";
use constant ACTUAL_DURATION_STR => "ActualDurationStr";
use constant ACTUAL_FINISH_DATE_STR => "actualfinish_str_1";
use constant ACTUAL_START_DATE_STR => "actualstart_str_1";
use constant ACTUAL_WORK_STR => "ActualWorkStr";
use constant ACWP_STR => "ACWPStr";
use constant BASELINE_COST_STR => "BaselineCostStr";
use constant BASELINE_DURATION_STR => "BaselineDurationStr";
use constant BASELINE_FINISH_DATE_STR => "baselinefinish_str_1";
use constant BASELINE_START_DATE_STR => "baselinestart_str_1";
use constant CURRENT_PLAN_START_DATE_STR => "curplanstart_str";
use constant CURRENT_PLAN_FINISH_DATE_STR => "curplanfinish_str";
use constant BASELINE_WORK_STR => "BaselineWorkStr";
use constant BCWP_STR => "BCWPStr";
use constant BCWS_STR => "BCWSStr";
use constant COST_STR => "CostStr";
use constant COST_VARIANCE_STR => "CostVarianceStr";
use constant FIXED_COST_STR => "FixedCostStr";
use constant PERCENT_COMPLETE_FK => "PercentCompleteStr";
use constant PERCENT_WORK_COMPLETE_FK => "PercentWorkCompleteStr";

use constant DBM_TABLE => "dbm_table";
use constant LAST_UPDATED => "last_updated";
use constant DEFAULT_DISPLAY_NAME => "default_display_name_1";



################
# main
################

# global variables
$currentDate; 
$currentTime;
getCurrentDateTime();

print "Start Time = $currentDate $currentTime \n";

$session = logon();


# If you wish to perform the calculation for a certain day, uncomment the
# following line and set it to a day with SQL date format, e.g., "1999-05-07";
#$currentDate = "1999-06-25";

$baselineFinish = "";
$actualFinish = "";
$collInstanceLogID = "";
$finish = "";

myMain();

getCurrentDateTime();

print "End Time = $currentDate $currentTime \n";


exit;

# 
# Initialize the values of the "pseudo constants".
#
sub initialize {

    # Get physical table names.  These names will be used in the query definitions.
    $PROJECT_MEASURE_TABLE = PjCLib->getPhysicalTableDbName("MSP_Task_Measure");
    $PROJECT_AGGREGAGE_MEASURE_TABLE = PjCLib->getPhysicalTableDbName("MSPAggregate_measure");

    $COLLECTIONDATE_TABLE = PjCLib->getPhysicalTableDbName("collectiondate");
    $CALENDARDATE_TABLE = PjCLib->getPhysicalTableDbName("calendardate");
    $PROJECTDATE_TABLE = PjCLib->getPhysicalTableDbName("ProjectDate");
    $ACTUALSTARTDATE_TABLE = PjCLib->getPhysicalTableDbName("ActualStart");
    $ACTUALFINISHDATE_TABLE = PjCLib->getPhysicalTableDbName("ActualFinish");
    $BASELINESTARTDATE_TABLE = PjCLib->getPhysicalTableDbName("BaselineStart");
    $BASELINEFINISHDATE_TABLE = PjCLib->getPhysicalTableDbName("BaselineFinish");
    $CURPLANSTARTDATE_TABLE = PjCLib->getPhysicalTableDbName("curplanstart");
    $CURPLANFINISHDATE_TABLE = PjCLib->getPhysicalTableDbName("curplanfinish");
    $MSP_MILESTONE_DIM = PjCLib->getPhysicalTableDbName("msp_milestone_dim");
    $MSP_EARNEDVALUE_MEASURE_TABLE = PjCLib->getPhysicalTableDbName("MSP_EarnedValue_Measure");

    # MSP Project Name dim table and field
    $PROJECTNAME_TABLE = PjCLib->getPhysicalTableDbName("mspprojectname_dim");
    $PROJECTNAME_FIELD = PjCLib->getPhysicalFieldDbName("mspprojectname_dim", "projectname");

    # Get physical field names for the task table.  These names will be used in the query definitions.
    $PROJECTNAME_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "projectname_fk");
    $TASK_NAME = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "taskname_fk");
    $TASK_ID = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "taskid_fk");
    $MEASURE = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "Number of Tasks");
    $ACTUAL_COST = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPActualCost");
    $ACTUAL_DURATION = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPActualDuration");
    $ACTUAL_FINISH_DATE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "actualfinish_fk");
    $ACTUAL_START_DATE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "actualstart_fk");
    $ACTUAL_WORK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPActualWork");
    $ACWP = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "ACWP");
    $BASELINE_COST = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPBaselineCost");
    $BASELINE_DURATION = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPBaselineDuration");
    $BASELINE_FINISH_DATE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "baselinefinish_fk");
    $BASELINE_START_DATE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "baselinestart_fk");
    $CURRENT_PLAN_START_DATE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "curplanstart_fk");
    $CURRENT_PLAN_FINISH_DATE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "curplanfinish_fk");
    $BASELINE_WORK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPBaselineWork");
    $BCWP = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "BCWP");
    $BCWS = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "BCWS");
    $COST = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "Cost_1");
    $COST_VARIANCE = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "CostVariance");
    $CPI = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "CPI");
    $SPI = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "SPI");
    $SV = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "SV");
    $CV = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "CV");
    $FIXED_COST = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "FixedCost_1");
    $PERCENT_COMPLETE = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPPercentComplete");
    $PERCENT_WORK_COMPLETE = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPPercentWorkComplete");
    $TASK_COMPLETE = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPTaskComplete");
    $ONTIME_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "ontime_fk");
    $MILESTONE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "milestone_fk");
    $COLLECTIONDATE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "collectiondate_fk");
    $COLLECTION_INSTANCE_ID = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "coll_instance_log_id");

    # Get physical field names for the aggregate table.  These names will be used in the query definitions.
    $MSP_AGGR_BP_TASK_COMPLETE = PjCLib->getPhysicalFieldDbName("MSPAggregate_measure", "MSP Aggr: BP Task Complete");
    $MSP_AGGR_CP_TASK_COMPLETE = PjCLib->getPhysicalFieldDbName("MSPAggregate_measure", "MSP Aggr: CP Task Complete");
    $MSP_AGGR_TASK_COMPLETE  = PjCLib->getPhysicalFieldDbName("MSPAggregate_measure", "MSP Aggr: Task Complete");
    $MSP_AGGR_COLLECTIONDATE_FK = PjCLib->getPhysicalFieldDbName("MSPAggregate_measure", "collectiondate_fk");
    $MSP_AGGR_CALENDARDATE_FK = PjCLib->getPhysicalFieldDbName("MSPAggregate_measure", "calendardate_fk");
    $MSP_AGGR_COLLECTION_INSTANCE_ID = PjCLib->getPhysicalFieldDbName("MSPAggregate_measure", "coll_instance_log_id");
    $MSP_AGGR_PROJECTDATE_FK = PjCLib->getPhysicalFieldDbName("MSPAggregate_measure", "projectdate_fk");

    # Get CQ Table and field names
    $CQ_MSP_EARNEDVALUE_MEASURE_TABLE = PjCLib->getCQTableDbName("MSP_EarnedValue_Measure");
    $CQ_PROJECT_ACWP_FIELD = PjCLib->getCQFieldDbName("MSP_EarnedValue_Measure", "Project ACWP");
    $CQ_PROJECT_BCWP_FIELD = PjCLib->getCQFieldDbName("MSP_EarnedValue_Measure", "Project BCWP");
    $CQ_PROJECT_BCWS_FIELD = PjCLib->getCQFieldDbName("MSP_EarnedValue_Measure", "Project BCWS");
    $CQ_PROJECT_CPI_FIELD = PjCLib->getCQFieldDbName("MSP_EarnedValue_Measure", "Project CPI");
    $CQ_PROJECT_SPI_FIELD = PjCLib->getCQFieldDbName("MSP_EarnedValue_Measure", "Project SPI");
    $CQ_PROJECTNAME_FIELD = PjCLib->getCQFieldDbName("MSP_EarnedValue_Measure", "projectname_fk");
    $CQ_COLLECTIONDATE_FIELD = PjCLib->getCQFieldDbName("MSP_EarnedValue_Measure", "collectiondate_fk");

    # These fields currently not used.
    #$TRANSFORMED = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "Transformed");
    #$MILESTONE = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "Milestone");
    #$MSP_AGGR_MILESTONE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "milestone_fk");

}

sub myMain {

    my $DBID;
    my $costStr;
    my $workStr;

    PjCLib->initializePjCLibrary($session, PjCLib->BOOLEAN_TRUE);
    initialize();

    print "Update Earned Value Aggregate table. Collection dated $currentDate.  Please wait...\n";



        #
        # Get measure table results
        #
        my $mainResultSet = buildTempProjTableResultSetRaw();
    
        #
        # Loop through each table and update the column values
        #
        while ($mainResultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
            $DBID = $mainResultSet->GetColumnValue(1);
            $acwp = $mainResultSet->GetColumnValue(2);
            $bcwp = $mainResultSet->GetColumnValue(3);
            $bcws = $mainResultSet->GetColumnValue(4);
            $collInstanceLogID = $mainResultSet->GetColumnValue(5);
            updateProjectMeasures($DBID, 
                                  $acwp,
                                  $bcwp,
                                  $bcws
                                  );

            $count = $count + 1;

        }
        print "Updated $count measure rows in table $PROJECT_MEASURE_TABLE.\n";

    updateLastUpdatedDate();
}


sub buildTempProjTableResultSetRaw {
    #
    # Return a result set containing the names of the time dimension tables
    # using raw SQL command
    #
    #print "buildTempProjTableResultSet started.\n";
                #" WHERE $PROJECT_MEASURE_TABLE.$PROJECTNAME_FK = $projectFK" .
    $queryStr = "SELECT dbid,$ACWP,$BCWP,$BCWS,$COLLECTION_INSTANCE_ID" .
                " FROM $PROJECT_MEASURE_TABLE" .
                " WHERE $COLLECTIONDATE_FK = (SELECT dbid FROM " .
                " collectiondate WHERE collectiondate_name = '$currentDate')";
    print "Execute query $queryStr\n";

    my $projResultSet = $session->BuildSQLQuery($queryStr);
    #print "Call Execute()\n";
    $projResultSet->Execute();

    #print "buildTempProjTableResultSet completed.\n";
    return $projResultSet;
}


sub replace {
    my ($text, $toReplace, $replaceWith,) = @_;

    #print "replace $text, $toReplace, $replaceWith\n";
    # replace all occurances

    my $notDone = 1;
    while($notDone) {
        my $index = index($text, $toReplace);
        if ($index >= 0) {
            substr($text, $index, length($toReplace), $replaceWith);
        } 
        else {
            $notDone = 0;
        }
    }
    #print "transformed to $text\n";
    return $text;
}


sub updateProjectMeasures {
    my ($DBID, 
        $acwp,
        $bcwp,
        $bcws
        ) = @_;

    #printf("ACWP = $acwp \n");
    #printf("BCWP = $bcwp \n");
    #printf("BCWS = $bcws \n\n");

    $sv = $bcwp - $bcws;
    $cv = $bcwp - $acwp;
    if ($acwp > 0.0) {
       $cpi = $bcwp / $acwp;
    } else {
       $cpi = 0.0;
    }

    if ($bcws > 0.0) {
       $spi = $bcwp / $bcws;
    } else {
       $spi = 0.0;
    }
    #printf("CV  = $cv \n");
    #printf("SV  = $sv \n");
    #printf("CPI  = $cpi \n");
    #printf("SPI  = $spi\n");


    #
    # Get the entity for this record.  Note that updates must be made
    # using an entity object.
    #
    my $entity = $session->GetEntityByDbId($PROJECT_MEASURE_TABLE, $DBID);


    #
    # Update the record
    #
    #print "Update record $DBID\n";
    $session->EditEntity($entity,"modify");
    $entity->SetFieldValue($CPI, "$cpi");
    $entity->SetFieldValue($SPI, "$spi");
    $entity->SetFieldValue($CV, "$cv");
    $entity->SetFieldValue($SV, "$sv");
    $retVal = $entity->Validate();
    #printf("entity->Validate() return value = $retVal\n");
    $entity->Commit();
    #printf("entity->Commit() return value = $retVal\n");

    #printf("EV Updated\n");


}
  
sub updateLastUpdatedDate() {
    
    print "updateLastUpdatedDate()...\n";
    $queryStr = "SELECT " . DBID .
                " FROM " . DBM_TABLE .
                " WHERE " . DEFAULT_DISPLAY_NAME . " = '". MSP_EARNEDVALUE_MEASURE . "';";
    print "Excute query $queryStr...\n";

    my $projResultSet = $session->BuildSQLQuery($queryStr);
    $projResultSet->Execute();
    #
    #
    #print "process eachrow...\n";
    if ($projResultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	$DBID = $projResultSet->GetColumnValue(1);

        my $entity = $session->GetEntityByDbId(DBM_TABLE, $DBID);
        print "Update record $DBID\n";
        $session->EditEntity($entity,"modify");
        $entity->SetFieldValue(LAST_UPDATED, $currentDate);
        $retVal = $entity->Validate();
        #printf("entity->Validate() return value = $retVal\n");
        $entity->Commit();
        #printf("entity->Commit() return value = $retVal\n");
    }
    print "updateLastUpdatedDate() completed.\n";

}



# get date and set the global variables $currentDate and $currentTime
sub getCurrentDateTime {
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);

    $mon = $mon + 1;
    $year = $year + 1900;

    if ($mday<10) {$mday="0$mday"; }  # add leading 0
    if ($mon<10) {$mon="0$mon"; }     # add leading 0

    # set the global variable $currentDate
    $currentDate="$year-$mon-$mday";

    # set the global variable $currentTime
    $currentTime = $hour . ":" . $min . ":" . $sec;

}

# MS Project date format is something like "Fri 2/19/99";
# This will convert the date to "1999-02-19";
sub convertMSProjectDate {
    ($_) = @_;

    if ($_ == "NA") {
      return "";
    }
    # remove spaces and useless day
    #$_ = $temp;
    s/^\s+//;  # kill leading spaces
    s/^(?:Sun|Mon|Tue|Wed|Thu|Fri|Sat)[a-z]*,?\s*//i; # kill useless weekday
    
    #printf("converting ". $_ . "\n");
    @array = split(/\//, $_);
    my ($mon,$day,$yr) = @array;
    if ($yr < 100) {
      $yr += 1900;
    }
    my $newDate = sprintf("%04d-%02d-%02d", $yr, $mon, $day);  # add leading 0 on month and day

    
    return $newDate;
}

sub zeroPad {
    my ($n,$Str) = @_;

    #
    # Simple routine to add leading zeros to a string
    #
    while (length($Str) < $n) { $Str = "0".$Str; }
    return $Str;
}



sub logon {

    #
    # Use the internal DashboardSystem login to get access to the Dashboard database
    #
    my $session = Win32::OLE->new('CLEARQUEST.SESSION');
    my $session = CQPerlExt::CQSession_Build();  
    $session->UserLogon("dashboardsystem", "dashboardsystem", "tstd1", "Dashboard");
    return $session;
}


#
# Return the DBID of the specified field value from specified table.
#
sub getDBID {
    my ($tableName, $fieldName, $fieldValue) = @_;
    my $DBID = "0";

    #if (length($fieldValue) == 0) {
    #    return $DBID";
    #}

    #print "buildTempProjTableResultSet started.\n";
    my $queryStr = "SELECT DBID FROM $tableName" .
                " WHERE $fieldName = '$fieldValue'";
    #print "getDBID(): Execute query $queryStr\n";

    my $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    while ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	$DBID = $resultSet->GetColumnValue(1);
    }
    return $DBID;
}


